@@ -13,7 +13,7 @@ import json
 import warnings
 
 from .. import model
-from ..model import effects, context, expressions, operators, properties, types
+from ..model import context, expressions, operators, properties, types, functions
 from ..utils import checks
 from ..metadata import version
 
@@ -25,20 +25,72 @@ _JANIMap = t.Dict[str, JSON]  # type: ignore
 
 
 class ModelFeature(enum.Enum):
+    """
+    An enum representing optional JANI model features.
+    """
+
     ARRAYS = "arrays"
+    """ Support for arrays. """
+
     DATATYPES = "datatypes"
+    """
+    Support for datatypes.
+    """
+
     DERIVED_OPERATORS = "derived-operators"
+    """
+    Support for derived operators.
+    """
+
     EDGE_PRIORITIES = "edge-priorities"
+    """
+    Support for edge priorities.
+    """
+
     FUNCTIONS = "functions"
+    """
+    Support for functions.
+    """
+
     HYPERBOLIC_FUNCTIONS = "hyperbolic-functions"
+    """
+    Support for hyperbolic functions.
+    """
+
     NAMED_EXPRESSIONS = "named-expressions"
-    NONDET_EXPRESSIONS = "nondet-expressions"
+    """
+    Support for named expressions.
+    """
+
+    NONDET_SELECTION = "nondet-selection"
+    """
+    Suport for non-deterministic selection expressions.
+    """
+
     STATE_EXIT_REWARDS = "state-exit-rewards"
+    """
+    Support for state exit rewards.
+    """
+
     TRADEOFF_PROPERTIES = "tradeoff-properties"
+    """
+    Support for tradeoff properties.
+    """
+
     TRIGONOMETRIC_FUNCTIONS = "trigonometric-functions"
+    """
+    Support for trigonometric functions.
+    """
 
     X_MOMBA_OPERATORS = "x-momba-operators"
+    """
+    Support for Momba non-standard operators.
+    """
+
     X_MOMBA_VALUE_PASSING = "x-momba-value-passing"
+    """
+    Support for Momba non-standard value passing.
+    """
 
 
 _NamedObject = t.Union[model.Location, model.Automaton]
@@ -138,7 +190,7 @@ checks.check_singledispatch(
 
 
 @functools.singledispatch
-def _dump_prop(prop: model.Property, ctx: JANIContext) -> JSON:
+def _dump_prop(prop: model.Expression, ctx: JANIContext) -> JSON:
     raise NotImplementedError(f"dump has not been implemented for property {prop}")
 
 
@@ -258,6 +310,38 @@ def _dump_derivative(expr: expressions.Derivative, ctx: JANIContext) -> JSON:
     return {"op": "der", "var": expr.identifier}
 
 
+@_dump_prop.register
+def _dump_array_access(expr: expressions.ArrayAccess, ctx: JANIContext) -> JSON:
+    ctx.require(ModelFeature.ARRAYS)
+    return {
+        "op": "aa",
+        "exp": _dump_prop(expr.array, ctx),
+        "index": _dump_prop(expr.index, ctx),
+    }
+
+
+@_dump_prop.register
+def _dump_array_value(expr: expressions.ArrayValue, ctx: JANIContext) -> JSON:
+    ctx.require(ModelFeature.ARRAYS)
+    return {
+        "op": "av",
+        "elements": list(_dump_prop(element, ctx) for element in expr.elements),
+    }
+
+
+@_dump_prop.register
+def _dump_array_constructor(
+    expr: expressions.ArrayConstructor, ctx: JANIContext
+) -> JSON:
+    ctx.require(ModelFeature.ARRAYS)
+    return {
+        "op": "av",
+        "var": expr.variable,
+        "length": _dump_prop(expr.length),
+        "exp": _dump_prop(expr.expression),
+    }
+
+
 @_dump_prop.register
 def _dump_sample(expr: expressions.Sample, ctx: JANIContext) -> JSON:
     return {
@@ -268,14 +352,24 @@ def _dump_sample(expr: expressions.Sample, ctx: JANIContext) -> JSON:
 
 @_dump_prop.register
 def _dump_selection(expr: expressions.Selection, ctx: JANIContext) -> JSON:
-    ctx.require(ModelFeature.NONDET_EXPRESSIONS)
+    ctx.require(ModelFeature.NONDET_SELECTION)
     return {
         "op": "nondet",
-        "var": expr.name,
+        "var": expr.variable,
         "exp": _dump_prop(expr.condition, ctx),
     }
 
 
+@_dump_prop.register
+def _dum_call(expr: functions.CallExpression, ctx: JANIContext) -> JSON:
+    ctx.require(ModelFeature.FUNCTIONS)
+    return {
+        "op": "call",
+        "function": expr.function,
+        "args": [_dump_prop(argument, ctx) for argument in expr.arguments],
+    }
+
+
 def _dump_prop_interval(pi: properties.Interval, ctx: JANIContext) -> JSON:
     prop_interval: _JANIMap = {}
     if pi.lower is not None:
@@ -405,20 +499,7 @@ def _dump_unary_path_formula(
     return jani_prop
 
 
-checks.check_singledispatch(_dump_prop, model.Property)
-
-
-@functools.singledispatch
-def _dump_target(target: effects.Target, ctx: JANIContext) -> JSON:
-    raise NotImplementedError(f"_dump_target has not been implemented for {target}")
-
-
-@_dump_target.register
-def _dump_target_identifier(target: effects.Name, ctx: JANIContext) -> JSON:
-    return target.identifier
-
-
-checks.check_singledispatch(_dump_target, effects.Target)
+checks.check_singledispatch(_dump_prop, model.Expression)
 
 
 def _dump_var_decl(decl: context.VariableDeclaration, ctx: JANIContext) -> JSON:
@@ -443,9 +524,9 @@ def _dump_const_decl(decl: context.ConstantDeclaration, ctx: JANIContext) -> JSO
     return jani_declaration
 
 
-def _dump_assignment(assignment: effects.Assignment, ctx: JANIContext) -> JSON:
+def _dump_assignment(assignment: model.Assignment, ctx: JANIContext) -> JSON:
     jani_assignment: _JANIMap = {
-        "ref": _dump_target(assignment.target, ctx),
+        "ref": _dump_prop(assignment.target, ctx),
         "value": _dump_prop(assignment.value, ctx),
     }
     if assignment.index != 0:
@@ -528,7 +609,7 @@ def _dump_action_pattern(
         #         "identifiers": list(pattern.identifiers),
         #     }
         # else:
-        return pattern.action_type.name
+        return pattern.action_type.label
     return None
 
 
@@ -573,7 +654,7 @@ def _dump_metadata(model_ctx: model.Context) -> _JANIMap:
 
 
 def _dump_action_type(action_type: model.ActionType, ctx: JANIContext) -> _JANIMap:
-    jani_action: _JANIMap = {"name": action_type.name}
+    jani_action: _JANIMap = {"name": action_type.label}
     jani_parameters: t.List[_JANIMap] = []
     for parameter in action_type.parameters:
         ctx.require(ModelFeature.X_MOMBA_VALUE_PASSING)
@@ -592,14 +673,14 @@ def dump_structure(
     network: model.Network,
     *,
     allow_momba_operators: bool = False,
-    properties: t.Optional[t.Mapping[str, model.Property]] = None,
+    properties: t.Optional[t.Mapping[str, model.Expression]] = None,
 ) -> JSON:
     ctx = JANIContext(allow_momba_operators=allow_momba_operators)
     jani_metadata: t.Dict[str, str] = {}
     if properties is None:
         properties = {
-            definition.name: definition.prop
-            for definition in network.ctx.named_properties.values()
+            definition.name: definition.expression
+            for definition in network.ctx.properties.values()
         }
     if "name" in network.ctx.metadata:
         jani_metadata
@@ -647,17 +728,30 @@ def dump_model(
     *,
     indent: t.Optional[int] = None,
     allow_momba_operators: bool = False,
-    properties: t.Optional[t.Mapping[str, model.Property]] = None,
-) -> bytes:
+    properties: t.Optional[t.Mapping[str, model.Expression]] = None,
+) -> str:
     """
-    Takes a Momba automata network and exports it to the JANI format.
+    Takes a Momba automaton :class:`~momba.model.Network` and returns a
+    JANI string representing the network.
 
-    Arguments:
-        network: The Momba automata network to export to JANI.
-        indent: Indentation of the final JSON.
+    The `indent` parameter controls the indentation of the resulting
+    JANI string.
+    `None` means no indentation.
+    Set `indent` to an integer, e.g., :code:`2`, to enable pretty formatting.
 
-    Returns:
-        The model in UTF-8 encoded JANI format.
+    Momba supports some non-standard operators which can either be preserved
+    or desugared into standard JANI.
+    This behavior is controlled via the flag `allow_momba_operators`.
+    If the flag is set to `True`, then the `x-momba-operators` JANI model
+    feature is enabled and the non-standard operators will be preserved
+    in the output.
+    Otherwise, if it is set to `False` (the default case), non-standard
+    operators are desugared into standard JANI.
+
+    The `properties` parameter allows to provide an mapping from :class:`str`
+    to :class:`~momba.model.Expression`.
+    The provided expressions will be put as additional properties into the
+    JANI-model output.
     """
     jani_structure = dump_structure(
         network, allow_momba_operators=allow_momba_operators, properties=properties
@@ -666,4 +760,4 @@ def dump_model(
         jani_structure,
         indent=indent,
         ensure_ascii=False,
-    ).encode("utf-8")
+    )
