@@ -12,7 +12,7 @@ import collections
 
 from mxu.maps import FrozenMap
 
-from . import actions, errors, expressions, types
+from . import actions, errors, expressions, observations, types
 
 if t.TYPE_CHECKING:
     from . import context
@@ -220,6 +220,7 @@ class Edge:
     guard: t.Optional[expressions.Expression] = None
     rate: t.Optional[expressions.Expression] = None
     annotation: t.Optional[Annotation] = None
+    observation: t.FrozenSet[observations.Observation] = frozenset()
 
     def create_edge_scope(self, parent: context.Scope) -> context.Scope:
         """
@@ -423,6 +424,7 @@ class Automaton:
         guard: t.Optional[expressions.Expression] = None,
         rate: t.Optional[expressions.Expression] = None,
         annotation: t.Optional[Annotation] = None,
+        observations: t.AbstractSet[observations.Observation] = frozenset(),
     ) -> None:
         """
         Creates an edge and adds it to the automaton.
@@ -438,6 +440,7 @@ class Automaton:
             guard,
             rate,
             FrozenMap(annotation),
+            frozenset(observations),
         )
         self.add_edge(edge)
 
@@ -489,7 +492,7 @@ class Automaton:
     def create_instance(
         self,
         *,
-        arguments: t.Sequence[expressions.Expression] = (),
+        arguments: t.Sequence[expressions.ValueOrExpression] = (),
         input_enable: t.AbstractSet[actions.ActionType] = frozenset(),
         comment: t.Optional[str] = None,
     ) -> Instance:
@@ -499,7 +502,9 @@ class Automaton:
         Passes the parameters to :class:`Instance`.
         """
         return Instance(
-            self, arguments=tuple(arguments), input_enable=frozenset(input_enable)
+            self,
+            arguments=tuple(map(expressions.ensure_expr, arguments)),
+            input_enable=frozenset(input_enable),
         )
 
 
